@@ -20,10 +20,13 @@ from string import Template
 
 import cridlib
 import pytz
+from babel.dates import format_date
 from configargparse import ArgumentParser
 from dateutil.relativedelta import relativedelta
 from iso3901 import ISRC
 from openpyxl import Workbook
+from openpyxl.styles import Border, Font, PatternFill, Side
+from tqdm import tqdm
 
 from .acrclient import ACRClient
 
@@ -38,7 +41,7 @@ Einhaltung der Einreichefrist für den Monat $month gemäss Ziffer 45 des Tarifs
 In der Sendungsmeldung enthalten sind die unter Buchstaben G (Verzeichnisse) des Tarifs
 genannten Programmangaben, in elektronischer Form. Die Angaben richten sich nach dem
 standardisierten Format gemäss der "Vorlage für Sendemeldungen: Gemeinsamer Tarif S" [2]
-sowie der Abstimmung zwischen SUSIA und $station_name.
+sowie der Abstimmung zwischen SUISA und $station_name.
 
 
 Diese Sendemeldung enthält unter anderem die unter Ziffer 34 geforderten Angaben:
@@ -57,7 +60,7 @@ Allfällige Beanstandungen oder technische Rückfragen zu dieser Sendemeldung m
 schriftlicher, elektronischer Form an $responsible_email gerichtet werden.
 
 Beanstandungen müssen, gemäss Ziffer 46, bis spätestens in drei Monaten vor dem $in_three_months
-erfolgen. Anschliessend gilt die Sendemeldung für $month als akzeptiert und die
+erfolgen. Anschliessend gilt die Sendemeldung für $month $year als akzeptiert und die
 Tarifbestimmungen als eingehalten.
 
 Korrekt und termingerecht gemeldete Beanstandungen werden unter Berücksichtigung der
@@ -68,11 +71,11 @@ Diese Email und Sendemeldung wurde automatisch generiert.
 Freundliche Grüsse,
 $station_name
 
---
-$email_footer
-
 [1] <https://www.suisa.ch/dam/jcr:1d82128b-8cc7-4cf0-b0ef-1890ae9d434d/GTS_2020-2024_GER.pdf>
 [2] <https://www.suisa.ch/dam/jcr:6934be98-4319-4565-8936-05007d83fd35/Vorlage_fuer_Sendemeldungen__GTS__2020-2022.xlsx>
+
+--
+$email_footer
 """
 
 
@@ -199,8 +202,8 @@ def get_arguments(parser: ArgumentParser):  # pragma: no cover
     parser.add_argument(
         "--email_subject",
         env_var="EMAIL_SUBJECT",
-        help="the subject of the email",
-        default="SUISA Sendemeldung",
+        help="The subject of the email, placeholders are $station_name, $year and $month",
+        default="SUISA Sendemeldung von $station_name für $year-$month",
     )
     parser.add_argument(
         "--email_text",
@@ -244,8 +247,15 @@ def get_arguments(parser: ArgumentParser):  # pragma: no cover
         "--timezone",
         env_var="TIMEZONE",
         help="set the timezone for localization",
+        required=True,
         default="UTC",
     )
+    parser.add_argument(
+        "--locale",
+        env_var="LOCALE",
+        help="set locale for date and time formatting",
+        default="de_CH",
+    )
     parser.add_argument(
         "--filename",
         env_var="FILENAME",
@@ -475,14 +485,17 @@ def get_csv(data, station_name=""):
     csv_writer = writer(csv, dialect="excel")
     csv_writer.writerow(header)
 
-    for entry in data:
+    for entry in tqdm(data, desc="preparing tracks for report"):
         metadata = entry.get("metadata")
         # parse timestamp
         timestamp = datetime.strptime(metadata.get("timestamp_local"), ACRClient.TS_FMT)
 
         ts_date = timestamp.strftime("%Y%m%d")
         ts_time = timestamp.strftime("%H:%M:%S")
-        duration = timedelta(seconds=metadata.get("played_duration"))
+        hours, remainder = divmod(metadata.get("played_duration"), 60 * 60)
+        minutes, seconds = divmod(remainder, 60)
+        # required format of duration field: hh:mm:ss
+        duration = f"{hours:02}:{minutes:02}:{seconds:02}"
 
         try:
             music = metadata.get("music")[0]
@@ -493,6 +506,8 @@ def get_csv(data, station_name=""):
         artist = get_artist(music)
 
         composer = ", ".join(music.get("contributors", {}).get("composers", ""))
+        if not composer:
+            composer = artist
 
         isrc = get_isrc(music)
         label = music.get("label")
@@ -565,6 +580,42 @@ def get_xlsx(data, station_name=""):
     for row in csv_reader:
         worksheet.append(row)
 
+    # the columns that should be styled as required (grey background)
+    required_columns = [
+        "Titel",
+        "Komponist",
+        "Interpret",
+        "Sendedatum",
+        "Sendedauer",
+        "Sendezeit",
+        "ISRC",
+        "Label",
+        "Label Code",
+        "Identifikationsnummer",
+    ]
+    font = Font(name="Calibri", bold=True, size=12)
+    side = Side(border_style="thick", color="000000")
+    border = Border(top=side, left=side, right=side, bottom=side)
+    fill = PatternFill("solid", bgColor="d9d9d9", fgColor="d9d9d9")
+    for cell in worksheet[1]:  # xlsx is 1-indexed
+        cell.font = font
+        cell.border = border
+        if cell.value in required_columns:
+            cell.fill = fill
+
+    # Try to approximate the required width by finding the longest values per column
+    dims = {}
+    for row in worksheet.rows:
+        for cell in row:
+            if cell.value:
+                dims[cell.column_letter] = max(
+                    (dims.get(cell.column_letter, 0), len(str(cell.value)))
+                )
+    # apply estimated width to each column
+    padding = 3
+    for col, value in dims.items():
+        worksheet.column_dimensions[col].width = value + padding
+
     workbook.save(xlsx)
     return xlsx
 
@@ -692,16 +743,26 @@ def main():  # pragma: no cover
     elif args.filetype == "csv":
         data = get_csv(data, station_name=args.station_name)
     if args.email:
-        email_subject = start_date.strftime(args.email_subject)
+        email_subject = Template(args.email_subject).substitute(
+            {
+                "station_name": args.station_name,
+                "year": format_date(start_date, format="yyyy", locale=args.locale),
+                "month": format_date(start_date, format="MM", locale=args.locale),
+            }
+        )
         # generate body
         text = Template(args.email_text).substitute(
             {
                 "station_name": args.station_name,
-                "month": start_date.strftime("%B"),
-                "year": start_date.strftime("%Y"),
-                "previous_year": (start_date - timedelta(days=365)).strftime("%Y"),
-                "in_three_months": (end_date + relativedelta(months=+3)).strftime(
-                    "%d. %B %Y"
+                "month": format_date(start_date, format="MMMM", locale=args.locale),
+                "year": format_date(start_date, format="yyyy", locale=args.locale),
+                "previous_year": format_date(
+                    start_date - timedelta(days=365), format="yyyy", locale=args.locale
+                ),
+                "in_three_months": format_date(
+                    datetime.now() + relativedelta(months=+3),
+                    format="long",
+                    locale=args.locale,
                 ),
                 "responsible_email": args.responsible_email,
                 "email_footer": args.email_footer,
